from typing import List, Dict
import datetime
import json

from qdata.errors import QdataError, ErrorCode
from . import common


def get_news_index(
    *,
    keywords_list: List[List[str]],
    start_date: str,
    end_date: str,
    cookies: str,
    area: int = 0
):
    return get_extended_index(
        keywords_list=keywords_list,
        start_date=start_date,
        end_date=end_date,
        cookies=cookies,
        area=area,
        type='news'
    )


def get_feed_index(
    *,
    keywords_list: List[List[str]],
    start_date: str,
    end_date: str,
    cookies: str,
    area: int = 0
):
    return get_extended_index(
        keywords_list=keywords_list,
        start_date=start_date,
        end_date=end_date,
        cookies=cookies,
        area=area,
        type='feed'
    )


def get_extended_index(
    *,
    keywords_list: List[List[str]],
    start_date: str,
    end_date: str,
    cookies: str,
    area: int,
    type: str
):
    if len(keywords_list) > 5:
        raise QdataError(ErrorCode.KEYWORD_LIMITED)
    for start_date, end_date in common.get_time_range_list(start_date, end_date):
        encrypt_json = common.get_encrypt_json(
            start_date=start_date,
            end_date=end_date,
            keywords=keywords_list,
            type=type,
            area=area,
            cookies=cookies
        )
        encrypt_datas = encrypt_json['data']['index']
        uniqid = encrypt_json['data']['uniqid']

        key = common.get_key(uniqid, cookies)
        for encrypt_data in encrypt_datas:
            encrypt_data['data'] = common.decrypt_func(key, encrypt_data['data'])
            for formated_data in format_data(encrypt_data):
                formated_data['type'] = type
                yield formated_data


def format_data(data: Dict):
    keyword = str(data['key'])
    start_date = datetime.datetime.strptime(data['startDate'], '%Y-%m-%d')
    end_date = datetime.datetime.strptime(data['endDate'], '%Y-%m-%d')
    date_list = []
    while start_date <= end_date:
        date_list.append(start_date)
        start_date += datetime.timedelta(days=1)

    index_datas = data['data']
    for i, cur_date in enumerate(date_list):
        try:
            index_data = index_datas[i]
        except IndexError:
            index_data = ''
        formated_data = {
            'keyword': [keyword_info['name'] for keyword_info in json.loads(keyword.replace('\'', '"'))],
            'date': cur_date.strftime('%Y-%m-%d'),
            'index': index_data if index_data else '0'
        }
        yield formated_data



import setuptools
import os

VERSION = '0.1.0'

CUR_DIR = os.path.abspath(os.path.dirname(__file__))
README = os.path.join(CUR_DIR, "README.md")
with open("README.md", "r") as fd:
    long_description = fd.read()

setuptools.setup(
    name="qdata",
    version=VERSION,
    description="Python SDK for getting data quickly",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/longxiaofei/spider-BaiduIndex",
    author="longxiaofei",
    author_email="libra.19951002@gmail.com",
    packages=setuptools.find_packages(
        exclude=["tests", "*.tests", "*.tests.*", "tests.*", "test.py"]
    ),
    install_requires=[
        "requests>=2.19.1"
    ],
    # entry_points={
    #     'console_scripts': [
    #         'tobe=tobe:main'
    #     ],
    # },
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        "Programming Language :: Python",
    ],
    keywords='data sdk',
    include_package_data=True
)



from typing import List, Dict
import datetime
import json

from . import common
from qdata.errors import QdataError, ErrorCode

ALL_KIND = ['all', 'pc', 'wise']


def get_search_index(
    *,
    keywords_list: List[List[str]],
    start_date: str,
    end_date: str,
    cookies: str,
    area: int = 0
):
    if len(keywords_list) > 5:
        raise QdataError(ErrorCode.KEYWORD_LIMITED)
    for start_date, end_date in common.get_time_range_list(start_date, end_date):
        encrypt_json = common.get_encrypt_json(
            start_date=start_date,
            end_date=end_date,
            keywords=keywords_list,
            type='search',
            area=area,
            cookies=cookies
        )
        encrypt_datas = encrypt_json['data']['userIndexes']
        uniqid = encrypt_json['data']['uniqid']

        key = common.get_key(uniqid, cookies)
        for encrypt_data in encrypt_datas:
            for kind in ALL_KIND:
                encrypt_data[kind]['data'] = common.decrypt_func(key, encrypt_data[kind]['data'])
            for formated_data in format_data(encrypt_data):
                yield formated_data


def format_data(data: Dict):
    """
        格式化堆在一起的数据
    """
    keyword = str(data['word'])
    start_date = datetime.datetime.strptime(data['all']['startDate'], '%Y-%m-%d')
    end_date = datetime.datetime.strptime(data['all']['endDate'], '%Y-%m-%d')
    date_list = []
    while start_date <= end_date:
        date_list.append(start_date)
        start_date += datetime.timedelta(days=1)

    for kind in ALL_KIND:
        index_datas = data[kind]['data']
        for i, cur_date in enumerate(date_list):
            try:
                index_data = index_datas[i]
            except IndexError:
                index_data = ''
            formated_data = {
                'keyword': [keyword_info['name'] for keyword_info in json.loads(keyword.replace('\'', '"'))],
                'type': kind,
                'date': cur_date.strftime('%Y-%m-%d'),
                'index': index_data if index_data else '0'
            }
            yield formated_data



from qdata.baidu_index import get_feed_index, get_news_index, get_search_index


keywords_list = [['张艺兴', '汪峰'], ['百度']]
cookies = """xxx"""


def test_get_feed_index():
    for index in get_feed_index(
        keywords_list=keywords_list,
        start_date='2018-01-01',
        end_date='2019-05-01',
        cookies=cookies
    ):
        print(index)


def test_get_news_index():
    for index in get_news_index(
        keywords_list=keywords_list,
        start_date='2018-01-01',
        end_date='2019-05-01',
        cookies=cookies
    ):
        print(index)


def test_get_search_index():
    for index in get_search_index(
        keywords_list=keywords_list,
        start_date='2018-01-01',
        end_date='2019-05-01',
        cookies=cookies
    ):
        print(index)


if __name__ == "__main__":
    test_get_feed_index()
    test_get_news_index()
    test_get_search_index()



# coding=utf-8
from typing import Dict, Iterator

import requests
from lxml import etree


headers = {
    'Connection': 'keep-alive',
    'Pragma': 'no-cache',
    'Cache-Control': 'no-cache',
    'Upgrade-Insecure-Requests': '1',
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 11_1_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.96 Safari/537.36',
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
    'Sec-Fetch-Site': 'same-origin',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-User': '?1',
    'Sec-Fetch-Dest': 'document',
    'Referer': 'https://www.baidu.com/',
    'Accept-Language': 'zh-CN,zh;q=0.9',
}


def get_url(url: str) -> str:
    """extract special url"""
    try:
        r = requests.get(url, allow_redirects=False)
        if r.status_code == 302 and 'Location' in r.headers.keys():
            return r.headers['Location']
    except Exception:
        pass
    return ''


def get_search(*, keyword: str, pn: int, cookies: str, domain: str = None) -> Iterator[Dict]:
    keyword = 'site: {} {}'.format(domain, keyword) if domain else keyword
    params = (
        ('wd', keyword),
        ('pn', (pn - 1) * 10),
        ('oq', keyword),
        ('tn', 'baiduhome_pg'),
        ('ie', 'utf-8'),
        ('rsv_idx', '2'),
        ('rsv_pq', 'd09ea91a000533ad'),
        ('rsv_t', 'a741enhrt8jcViHd/8Q+gb0DnCzjIbctyKmpOkRk6BibYwnyQXvHFSqrZtTKeUHQlE4s'),
    )
    response = requests.get('https://www.baidu.com/s', headers=headers, params=params)
    html = etree.HTML(response.text)
    items = html.xpath('//*/h3/a')
    for item in items:
        title = ''.join(item.xpath('.//text()'))
        href = item.xpath('./@href')[0]
        url = get_url(href)
        if not url:
            continue
        if domain and domain not in url:
            continue
        yield {'title': title, 'url': url}


def get_all_search(*, keyword: str, cookies: str, domain: str = None) -> Iterator[Dict]:
    for pn in range(1, 76):
        for item in get_search(
            keyword=keyword,
            pn=pn,
            cookies=compile,
            domain=domain
        ):
            yield item



from typing import List, Dict, Tuple
from urllib.parse import urlencode
import math
import datetime
import json

from qdata.errors import ErrorCode, QdataError

import requests


headers = {
    'Host': 'index.baidu.com',
    'Connection': 'keep-alive',
    'X-Requested-With': 'XMLHttpRequest',
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36',
}


def get_time_range_list(startdate: str, enddate: str) -> List[Tuple[str, str]]:
    """
        切分时间段
    """
    date_range_list = []
    startdate = datetime.datetime.strptime(startdate, '%Y-%m-%d')
    enddate = datetime.datetime.strptime(enddate, '%Y-%m-%d')
    while 1:
        tempdate = startdate + datetime.timedelta(days=300)
        if tempdate > enddate:
            date_range_list.append((startdate, enddate))
            break
        date_range_list.append((startdate, tempdate))
        startdate = tempdate + datetime.timedelta(days=1)
    return date_range_list


def split_keywords(keywords: List) -> List[List[str]]:
    """
    一个请求最多传入5个关键词, 所以需要对关键词进行切分
    """
    return [keywords[i*5: (i+1)*5] for i in range(math.ceil(len(keywords)/5))]


def http_get(url: str, cookies: str) -> str:
    """
        发送get请求, 程序中所有的get都是调这个方法
        如果想使用多cookies抓取, 和请求重试功能
        在这自己添加
    """
    _headers = headers.copy()
    _headers['Cookie'] = cookies
    try:
        response = requests.get(url, headers=_headers, timeout=5)
    except requests.Timeout:
        raise QdataError(ErrorCode.NETWORK_ERROR)
    if response.status_code != 200:
        raise QdataError(ErrorCode.NETWORK_ERROR)
    return response.text


def get_key(uniqid: str, cookies: str) -> str:
    url = 'http://index.baidu.com/Interface/api/ptbk?uniqid=%s' % uniqid
    html = http_get(url, cookies)
    datas = json.loads(html)
    key = datas['data']
    return key


def decrypt_func(key: str, data: str) -> List[str]:
    """
        数据解密方法
    """
    a = key
    i = data
    n = {}
    s = []
    for o in range(len(a)//2):
        n[a[o]] = a[len(a)//2 + o]
    for r in range(len(data)):
        s.append(n[i[r]])
    return ''.join(s).split(',')


def get_encrypt_json(
    *,
    start_date: str,
    end_date: str,
    keywords: List[List[str]],
    type: str,
    area: int,
    cookies: str
) -> Dict:
    pre_url_map = {
        'search': 'http://index.baidu.com/api/SearchApi/index?',
        'news': 'http://index.baidu.com/api/NewsApi/getNewsIndex?',
        'feed': 'http://index.baidu.com/api/FeedSearchApi/getFeedIndex?'
    }

    pre_url = pre_url_map[type]
    word_list = [
        [{'name': keyword, 'wordType': 1} for keyword in keyword_list]
        for keyword_list in keywords
    ]
    request_args = {
        'word': json.dumps(word_list),
        'startDate': start_date.strftime('%Y-%m-%d'),
        'endDate': end_date.strftime('%Y-%m-%d'),
        'area': area
    }
    url = pre_url + urlencode(request_args)
    html = http_get(url, cookies)
    datas = json.loads(html)
    if datas['status'] == 10000:
        raise QdataError(ErrorCode.NO_LOGIN)
    if datas['status'] != 0:
        raise QdataError(ErrorCode.UNKNOWN)
    return datas


def test_cookies(cookies: str) -> bool:
    """
        测试cookie是否可用
    """
    html = http_get('https://www.baidu.com/', cookies)
    return '退出' in html



from enum import Enum


class ErrorCode(int, Enum):
    UNKNOWN = 10002
    NETWORK_ERROR = 10003

    NO_LOGIN = 20000
    KEYWORD_LIMITED = 20001


CODE_MSG_MAP = {
    ErrorCode.NO_LOGIN: 'cookies失效，请重新获取cookies',
    ErrorCode.UNKNOWN: '未知错误',
    ErrorCode.NETWORK_ERROR: '网络错误',
    ErrorCode.KEYWORD_LIMITED: ('关键词最多传递5个, '
                                '可以使用`from qdata.baidu_index.common import split_keywords`,'
                                '对关键词进行切分')
}


class QdataError(Exception):
    def __init__(self, code: ErrorCode):
        self.code = code
        self.msg = CODE_MSG_MAP.get(code)

    def __str__(self):
        return repr(f"ERROR-{self.code}: {self.msg}")



from qdata.baidu_search import get_all_search


def test_all_search():
    _keyword = '大碗面'
    _domain = 'dianping.com'
    _cookies = '这里放cookie'
    for result in get_all_search(keyword=_keyword, cookies=_cookies, domain=_domain):
        print(result)


if __name__ == "__main__":
    test_all_search()
